package server.command.commands;

import java.security.MessageDigest;
import java.security.PublicKey;

import common.color.TextColor;
import common.network.IPlayer;
import common.util.EncryptUtil;
import common.util.Pair;
import server.command.Command;
import server.command.CommandEnvironment;
import server.command.Executor;
import server.command.RunException;
import server.network.Player;
import server.util.Form;

public class CommandPubkey extends Command {
	public CommandPubkey() {
		super("pubkey");

		this.addPlayer("player", true);
		this.setParamsOptional();
		this.addString("keySpec", false);
		this.addString("keyData", false);
		this.addString("keyHash", false);
		this.addString("keyName", false);
	}
	
	public void exec(CommandEnvironment env, Executor exec, Player player, String keySpec, String keyData, String keyHash, String keyName) {
		if(exec.isPlayer()) {
			if(keySpec != null || keyData != null || keyHash != null || keyName != null)
				throw new RunException("Bei Verwendung als Spieler darf kein Schlüssel angegeben werden");
			if(player.getAdmin() && player != exec)
				throw new RunException("%s ist ein Admin", player.getUser());
			((Player)exec).displayForm(new Form() {
				private Field checkField;
				private Field keyField;
				
				protected void init() {
					this.checkField = player.getPasswordHash() == null || player != exec ? null : this.addPassword("Aktuelles Passwort", 0, IPlayer.MAX_PASS_LENGTH, "");
					this.keyField = this.addField("Neuer Pubkey", 1, 960, "");
				}
				
				public String getTitle() {
					return "Schlüssel für " + player.getUser() + " ändern";
				}
				
				protected void accept() {
					Player plr = env.getServer().getPlayer(player.getUser());
					if(!((Player)exec).isAdmin() || plr == null || (plr.isAdmin() && plr != exec)) {
						exec.logConsole(TextColor.DRED + "Ein Fehler ist aufgetreten");
						return;
					}
					if(this.checkField != null && plr.getPasswordHash() != null && !MessageDigest.isEqual(EncryptUtil.hashPassword(this.checkField.get(), plr.getPasswordHash().second()), plr.getPasswordHash().first())) {
						exec.logConsole(TextColor.RED + "Falsches Passwort eingegeben");
						return;
					}
					Pair<PublicKey, String> key;
					try {
						key = EncryptUtil.parseArmoredPubkey(this.keyField.get());
					}
					catch(IllegalArgumentException e) {
						exec.logConsole(TextColor.RED + "Ungültiger Schlüssel");
						return;
					}
					plr.setPasswordHash(null);
					plr.setPubkey(key.first());
					exec.logConsole(TextColor.GREEN + "Schlüssel" + (plr != exec ? " für %s" : "") + " gesetzt", plr.getUser());
				}
			});
		}
		else if(exec.isConsole()) {
			if(keySpec == null || keyData == null || keyHash == null)
				throw new RunException("Bei Verwendung in der Konsole muss ein Schlüssel angegeben werden");
			Pair<PublicKey, String> key;
			try {
				key = EncryptUtil.parseArmoredPubkey(keySpec + " " + keyData + " " + keyHash + (keyName == null ? null : " " + keyName));
			}
			catch(IllegalArgumentException e) {
				throw new RunException(e, "Ungültiger Schlüssel");
			}
			player.setPasswordHash(null);
			player.setPubkey(key.first());
			exec.logConsole(TextColor.GREEN + "Schlüssel für %s gesetzt", player.getUser());
		}
	}
}
